/*
    Copyright (C) 2002 Terratron Technologies Inc.  All Rights Reserved.
    Author: Steven Osman

    This file is part of sps2.

    Before using this file you MUST agree to the license agreement in
    the file LICENSE provided with this package.

    For more information visit:
    http://www.playstation2-linux.com/projects/sps2/
    http://window.terratron.com/~sosman/ps2linux/
    http://www.terratron.com/
*/
#ifndef __SPS2_MOD_H
#define __SPS2_MOD_H

#include <asm/uaccess.h>
#include <linux/list.h>
#include <linux/vmalloc.h>
#include <asm/io.h>
#include <linux/mm.h>

#include "sps2interface.h"

#define SPS2_DEFAULT_VALIDATE_DMA_SIZE (16*1024*1024)

/** 
 * Stolen from linux 2.4 kernel.
 * Allows you to loop through a list and even delete elements.
 */
#ifndef list_for_each_safe
#define list_for_each_safe(pos, n, head) \
        for (pos = (head)->next, n = pos->next; pos != (head); \
                pos = n, n = pos->next)
#endif

#ifndef list_for_each
#define list_for_each(pos, head) \
         for (pos = (head)->next; pos != (head); pos = pos->next)
#endif

// The following constants are used for the physical memory map
#define MEMORY_FREE     0
#define MEMORY_START    1
#define MEMORY_CONTINUE 2
#define MEMORY_MASK     3

/**
 * This structure will be shared across all open instances of this
 * module for a given process.
 * Physical memory map is a 2048 byte map of all physical memory.  Each
 * 4096 byte page is represented by two bits.
 * 00 = unallocated by this process
 * 01 = beginning of contiguous area
 * 10 = continuation of contiguous area
 * 11 = unused
 */
typedef struct {
  int iReferenceCounter;

  unsigned char aucMap[2048];
} physicalMemoryMap_t;

/**
 * This structure will be stored as the private_data field
 * for our file.
 */
typedef struct {
  // These should always be the first four elements of this structure
  int iVersionMajor; // The major version we will be compatible with
  int iVersionMinor; // The minor version we will be compatible with
  int iShare;        // This instance is willing to share with others

  physicalMemoryMap_t *pPhysicalMemoryMap;

  struct list_head mapList;
  int iMapCount;

} fileData_t;

/**
 * This structure is a member of the fileData->mapList list.  Basically
 * for each mmap() call, an entry is created.
 */
typedef struct {
  struct list_head list;
  unsigned long ulStart;
  int iBlockCount;
  int iBlockSize;
  int iBlockOrder;
  int *piReferenceCounter;

  enum {
    MAP_TYPE_MEMORY=0,
    MAP_TYPE_EE_REGISTERS,
    MAP_TYPE_VU_MEMORY,
    MAP_TYPE_GS_REGISTERS,
  } mapType;

  unsigned long aulBlocks;
} memoryMap_t;

#endif
